# Reference

<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

### Defined types

* [`sqlserver::config`](#sqlserverconfig): Define Resource Type: sqlserver::config
* [`sqlserver::database`](#sqlserverdatabase): Define Resource Type: sqlserver::database
* [`sqlserver::login`](#sqlserverlogin): Define Resource Type: sqlserver::login
* [`sqlserver::login::permissions`](#sqlserverloginpermissions): Define Resource Type: sqlserver::login::permissions#
* [`sqlserver::role`](#sqlserverrole): Define Resource Type: sqlserver::role::permissions
* [`sqlserver::role::permissions`](#sqlserverrolepermissions): Define Resource Type: sqlserver::role::permissions
* [`sqlserver::sp_configure`](#sqlserversp_configure): Defined Resource Type: sqlserver::sp_configure
* [`sqlserver::user`](#sqlserveruser): Define Resource Type: sqlserver::user
* [`sqlserver::user::permissions`](#sqlserveruserpermissions): Define Resource Type: sqlserver::user::permissions

### Resource types

* [`sqlserver_features`](#sqlserver_features): Due to our prefetch and unaware of what name the user will provide we munge the value to meet our expecations.
* [`sqlserver_instance`](#sqlserver_instance): Namevar
* [`sqlserver_tsql`](#sqlserver_tsql): Namevar

### Functions

* [`sqlserver_is_domain_or_local_user`](#sqlserver_is_domain_or_local_user)
* [`sqlserver_upcase`](#sqlserver_upcase)
* [`sqlserver_validate_hash_uniq_values`](#sqlserver_validate_hash_uniq_values)
* [`sqlserver_validate_instance_name`](#sqlserver_validate_instance_name)
* [`sqlserver_validate_range`](#sqlserver_validate_range)
* [`sqlserver_validate_size`](#sqlserver_validate_size)

### Tasks

* [`get_sql_logins`](#get_sql_logins): Retrieve information about the logins configured for a SQL Server instance.
* [`get_sqlagent_jobs`](#get_sqlagent_jobs): Return information about SQL Agent jobs and job steps.
* [`set_sql_logins`](#set_sql_logins): Set IsDisabled, and Password properties of a SQL Login
* [`start_sql_agent_job`](#start_sql_agent_job): Start SQL Agent jobs on a server. You can start at a specified job step number (zero based indexes), and you can either wait on the job to co

## Defined types

### <a name="sqlserverconfig"></a>`sqlserver::config`

Define Resource Type: sqlserver::config

#### Examples

##### 

```puppet
sqlserver::config{'MSSQLSERVER':
  admin_user => 'sa',
  admin_pass => 'PuppetP@ssword1',
}
```

#### Parameters

The following parameters are available in the `sqlserver::config` defined type:

* [`instance_name`](#instance_name)
* [`admin_user`](#admin_user)
* [`admin_pass`](#admin_pass)
* [`admin_login_type`](#admin_login_type)

##### <a name="instance_name"></a>`instance_name`

Data type: `String[1,16]`

The instance name you want to manage.  Defaults to the $title when not defined explicitly.

Default value: `$title`

##### <a name="admin_user"></a>`admin_user`

Data type: `Optional[String]`

Only required for SQL_LOGIN type. A user/login who has sysadmin rights on the server

Default value: `''`

##### <a name="admin_pass"></a>`admin_pass`

Data type: `Optional[String]`

Only required for SQL_LOGIN type. The password in order to access the server to be managed.

Default value: `''`

##### <a name="admin_login_type"></a>`admin_login_type`

Data type: `Enum['SQL_LOGIN', 'WINDOWS_LOGIN']`

The type of account use to configure the server.  Valid values are SQL_LOGIN and WINDOWS_LOGIN, with a default of SQL_LOGIN
The SQL_LOGIN requires the admin_user and admin_pass to be set
The WINDOWS_LOGIN requires the adm_user and admin_pass to be empty or undefined

Default value: `'SQL_LOGIN'`

### <a name="sqlserverdatabase"></a>`sqlserver::database`

Requirement/Dependencies:
Requires defined type {sqlserver::config} in order to execute against the SQL Server instance

* **See also**
  * http://msdn.microsoft.com/en-us/library/ff929071.aspx
    * Contained Databases
  * http://msdn.microsoft.com/en-us/library/ms176061.aspx
    * CREATE DATABASE TSQL
  * http://msdn.microsoft.com/en-us/library/ms174269.aspx
    * ALTER DATABASE TSQL
  * http://msdn.microsoft.com/en-us/library/ms190303.aspx
    * System Languages

#### Parameters

The following parameters are available in the `sqlserver::database` defined type:

* [`db_name`](#db_name)
* [`instance`](#instance)
* [`ensure`](#ensure)
* [`compatibility`](#compatibility)
* [`collation_name`](#collation_name)
* [`filestream_non_transacted_access`](#filestream_non_transacted_access)
* [`filestream_directory_name`](#filestream_directory_name)
* [`filespec_name`](#filespec_name)
* [`filespec_filename`](#filespec_filename)
* [`filespec_size`](#filespec_size)
* [`filespec_maxsize`](#filespec_maxsize)
* [`filespec_filegrowth`](#filespec_filegrowth)
* [`log_name`](#log_name)
* [`log_filename`](#log_filename)
* [`log_size`](#log_size)
* [`log_maxsize`](#log_maxsize)
* [`log_filegrowth`](#log_filegrowth)
* [`containment`](#containment)
* [`default_fulltext_language`](#default_fulltext_language)
* [`default_language`](#default_language)
* [`nested_triggers`](#nested_triggers)
* [`transform_noise_words`](#transform_noise_words)
* [`two_digit_year_cutoff`](#two_digit_year_cutoff)
* [`db_chaining`](#db_chaining)
* [`trustworthy`](#trustworthy)

##### <a name="db_name"></a>`db_name`

Data type: `String[1,128]`

The database you would like to manage

Default value: `$title`

##### <a name="instance"></a>`instance`

Data type: `String[1,16]`

The name of the instance which to connect to, instance names can not be longer than 16 characters

Default value: `'MSSQLSERVER'`

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present', 'absent']`

Defaults to 'present', valid values are 'present' | 'absent'

Default value: `'present'`

##### <a name="compatibility"></a>`compatibility`

Data type: `Integer`

Numberic representation of what SQL Server version you want the database to be compatabible with.

Default value: `100`

##### <a name="collation_name"></a>`collation_name`

Data type: `Optional[String[1]]`



Default value: ``undef``

##### <a name="filestream_non_transacted_access"></a>`filestream_non_transacted_access`

Data type: `Optional[Enum['OFF', 'READ_ONLY', 'FULL']]`

Value should be { OFF | READ_ONLY | FULL }
Specifies the level of non-transactional FILESTREAM access to the database.

Default value: ``undef``

##### <a name="filestream_directory_name"></a>`filestream_directory_name`

Data type: `Optional[Pattern[/^[\w|\s]+$/]]`

A windows-compatible directory name.
This name should be unique among all the Database_Directory names in the SQL Server instance.
Uniqueness comparison is case-insensitive, regardless of SQL Server collation settings.
This option should be set before creating a FileTable in this database.

Default value: ``undef``

##### <a name="filespec_name"></a>`filespec_name`

Data type: `Optional[String[1,128]]`

Specifies the logical name for the file.
NAME is required when FILENAME is specified, except when specifying one of the FOR ATTACH clauses.
A FILESTREAM filegroup cannot be named PRIMARY.

Default value: ``undef``

##### <a name="filespec_filename"></a>`filespec_filename`

Data type: `Optional[Stdlib::Absolutepath]`

Specifies the operating system (physical) file name.

Default value: ``undef``

##### <a name="filespec_size"></a>`filespec_size`

Data type: `Optional[String[1]]`

Specifies the size of the file. The kilobyte (KB), megabyte (MB), gigabyte (GB), or terabyte (TB) suffixes can be used.
The default is MB.
Values can not be greater than 2147483647

Default value: ``undef``

##### <a name="filespec_maxsize"></a>`filespec_maxsize`

Data type: `Optional[String[1]]`

Specifies the maximum size to which the file can grow. MAXSIZE cannot be specified when the os_file_name is specified as a UNC path

Default value: ``undef``

##### <a name="filespec_filegrowth"></a>`filespec_filegrowth`

Data type: `Optional[String[1]]`

Specifies the automatic growth increment of the file.
The FILEGROWTH setting for a file cannot exceed the MAXSIZE setting.
FILEGROWTH cannot be specified when the os_file_name is specified as a UNC path.
FILEGROWTH does not apply to a FILESTREAM filegroup.

Default value: ``undef``

##### <a name="log_name"></a>`log_name`

Data type: `Optional[String[1,128]]`

Specifies the logical name for the file.
NAME is required when FILENAME is specified, except when specifying one of the FOR ATTACH clauses.
A FILESTREAM filegroup cannot be named PRIMARY.

Default value: ``undef``

##### <a name="log_filename"></a>`log_filename`

Data type: `Optional[Stdlib::Absolutepath]`

Specifies the operating system (physical) file name.

Default value: ``undef``

##### <a name="log_size"></a>`log_size`

Data type: `Optional[String[1]]`

Specifies the size of the file.
The kilobyte (KB), megabyte (MB), gigabyte (GB), or terabyte (TB) suffixes can be used.
The default is MB.  Values can not be greater than 2147483647

Default value: ``undef``

##### <a name="log_maxsize"></a>`log_maxsize`

Data type: `Optional[String[1]]`

Specifies the maximum size to which the file can grow. MAXSIZE cannot be specified when the os_file_name is specified as a UNC path

Default value: ``undef``

##### <a name="log_filegrowth"></a>`log_filegrowth`

Data type: `Optional[String[1]]`

Specifies the automatic growth increment of the file.
The FILEGROWTH setting for a file cannot exceed the MAXSIZE setting.
FILEGROWTH cannot be specified when the os_file_name is specified as a UNC path.
FILEGROWTH does not apply to a FILESTREAM filegroup.

Default value: ``undef``

##### <a name="containment"></a>`containment`

Data type: `Enum['PARTIAL', 'NONE']`

Defaults to 'NONE'.
Other possible values are 'PARTIAL', see http://msdn.microsoft.com/en-us/library/ff929071.aspx

Default value: `'NONE'`

##### <a name="default_fulltext_language"></a>`default_fulltext_language`

Data type: `String[1]`

Language name i.e. us_english which are documented at http://msdn.microsoft.com/en-us/library/ms190303.aspx

Default value: `'English'`

##### <a name="default_language"></a>`default_language`

Data type: `String[1]`

Language name i.e. us_english which are documented at http://msdn.microsoft.com/en-us/library/ms190303.aspx

Default value: `'us_english'`

##### <a name="nested_triggers"></a>`nested_triggers`

Data type: `Optional[Enum['ON', 'OFF']]`

On | Off  see http://msdn.microsoft.com/en-us/library/ms178101.aspx

Default value: ``undef``

##### <a name="transform_noise_words"></a>`transform_noise_words`

Data type: `Optional[Enum['ON', 'OFF']]`

ON | OFF

Default value: ``undef``

##### <a name="two_digit_year_cutoff"></a>`two_digit_year_cutoff`

Data type: `Integer[1753, 9999]`

Defaults to 2049 | <any year between 1753 and 9999>

Default value: `2049`

##### <a name="db_chaining"></a>`db_chaining`

Data type: `Enum['ON', 'OFF']`

ON | OFF
When ON is specified, the database can be the source or target of a cross-database ownership chain.
When OFF, the database cannot participate in cross-database ownership chaining. The default is OFF.

Default value: `'OFF'`

##### <a name="trustworthy"></a>`trustworthy`

Data type: `Enum['ON', 'OFF']`

When ON is specified, database modules that use an impersonation context can access resources outside the database.
For example, views, user-defined functions, or stored procedures.
When OFF, database modules in an impersonation context cannot access resources outside the database. The default is OFF.

Default value: `'OFF'`

### <a name="sqlserverlogin"></a>`sqlserver::login`

Requirement/Dependencies:
Requires defined type {sqlserver::config} in order to execute against the SQL Server instance

* **See also**
  * Puppet::Parser::Fucntions#sqlserver_validate_instance_name
  * http://msdn.microsoft.com/en-us/library/ms186320(v=sql.110).aspx
    * Server Role Members
  * http://technet.microsoft.com/en-us/library/ms189751(v=sql.110).aspx
    * Create Login
  * http://technet.microsoft.com/en-us/library/ms189828(v=sql.110).aspx
    * Alter Login

#### Parameters

The following parameters are available in the `sqlserver::login` defined type:

* [`login`](#login)
* [`instance`](#instance)
* [`ensure`](#ensure)
* [`password`](#password)
* [`svrroles`](#svrroles)
* [`login_type`](#login_type)
* [`default_database`](#default_database)
* [`default_language`](#default_language)
* [`check_expiration`](#check_expiration)
* [`check_policy`](#check_policy)
* [`disabled`](#disabled)
* [`permissions`](#permissions)

##### <a name="login"></a>`login`

Data type: `Any`

The SQL or Windows login you would like to manage

Default value: `$title`

##### <a name="instance"></a>`instance`

Data type: `String[1,16]`

The name of the instance which to connect to, instance names can not be longer than 16 characters

Default value: `'MSSQLSERVER'`

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present', 'absent']`

Defaults to 'present', valid values are 'present' | 'absent'

Default value: `'present'`

##### <a name="password"></a>`password`

Data type: `Optional[String]`

Plain text password. Only applicable when Login_Type = 'SQL_LOGIN'.

Default value: ``undef``

##### <a name="svrroles"></a>`svrroles`

Data type: `Optional[Hash]`

A hash of preinstalled server roles that you want assigned to this login.
sample usage would be  { 'diskadmin' => 1, 'dbcreator' => 1, 'sysadmin' => 0,  }

Default value: `{ }`

##### <a name="login_type"></a>`login_type`

Data type: `Enum['SQL_LOGIN', 'WINDOWS_LOGIN']`

Defaults to 'SQL_LOGIN', possible values are 'SQL_LOGIN' or 'WINDOWS_LOGIN'

Default value: `'SQL_LOGIN'`

##### <a name="default_database"></a>`default_database`

Data type: `String`

The database that when connecting the login should default to, the default value is 'master'

Default value: `'master'`

##### <a name="default_language"></a>`default_language`

Data type: `String`

The default language is 'us_english', a list of possible

Default value: `'us_english'`

##### <a name="check_expiration"></a>`check_expiration`

Data type: `Boolean`

Default value is false, possible values of true | false. Only applicable when Login_Type = 'SQL_LOGIN'.

Default value: ``false``

##### <a name="check_policy"></a>`check_policy`

Data type: `Boolean`

Default value is false, possible values are true | false. Only applicable when Login_Type = 'SQL_LOGIN'.

Default value: ``true``

##### <a name="disabled"></a>`disabled`

Data type: `Boolean`

Default value is false.  Accepts [Boolean] values of true or false.

Default value: ``false``

##### <a name="permissions"></a>`permissions`

Data type: `Optional[Hash]`

A hash of permissions that should be managed for the login.
Valid keys are 'GRANT', 'GRANT_WITH_OPTION', 'DENY' or 'REVOKE'.
Valid values must be an array of Strings i.e. {'GRANT' => ['CONNECT SQL', 'CREATE ANY DATABASE'] }

Default value: `{ }`

### <a name="sqlserverloginpermissions"></a>`sqlserver::login::permissions`

Requirement/Dependencies:
Requires defined type {sqlserver::config} in order to execute against the SQL Server instance

#### Parameters

The following parameters are available in the `sqlserver::login::permissions` defined type:

* [`login`](#login)
* [`permissions`](#permissions)
* [`state`](#state)
* [`instance`](#instance)
* [`with_grant_option`](#with_grant_option)

##### <a name="login"></a>`login`

Data type: `String[1,128]`

The login for which the permission will be manage.

##### <a name="permissions"></a>`permissions`

Data type: `Array[String[4,128]]`

An array of permissions you would like managed. i.e. ['SELECT', 'INSERT', 'UPDATE', 'DELETE']

##### <a name="state"></a>`state`

Data type: `Pattern[/(?i)^(GRANT|REVOKE|DENY)$/]`

The state you would like the permission in.
Accepts 'GRANT', 'DENY', 'REVOKE'.
Please note that REVOKE equates to absent and will default to database and system level permissions.

Default value: `'GRANT'`

##### <a name="instance"></a>`instance`

Data type: `String[1,16]`

The name of the instance where the user and database exists. Defaults to 'MSSQLSERVER'

Default value: `'MSSQLSERVER'`

##### <a name="with_grant_option"></a>`with_grant_option`

Data type: `Optional[Boolean]`

Bolean value that allows user to grant options.

Default value: ``false``

### <a name="sqlserverrole"></a>`sqlserver::role`

Requirement/Dependencies:
Requires defined type {sqlserver::config} in order to execute against the SQL Server instance

#### Parameters

The following parameters are available in the `sqlserver::role` defined type:

* [`ensure`](#ensure)
* [`role`](#role)
* [`instance`](#instance)
* [`authorization`](#authorization)
* [`type`](#type)
* [`database`](#database)
* [`permissions`](#permissions)
* [`members`](#members)
* [`members_purge`](#members_purge)

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present', 'absent']`

Whether the role should be absent or present

Default value: `'present'`

##### <a name="role"></a>`role`

Data type: `String[1,128]`

The name of the role for which the permissions will be manage.

Default value: `$title`

##### <a name="instance"></a>`instance`

Data type: `String[1,16]`

The name of the instance where the role and database exists. Defaults to 'MSSQLSERVER'

Default value: `'MSSQLSERVER'`

##### <a name="authorization"></a>`authorization`

Data type: `Optional[String]`

The database principal that should own the role

Default value: ``undef``

##### <a name="type"></a>`type`

Data type: `Enum['SERVER', 'DATABASE']`

Whether the Role is `SERVER` or `DATABASE`

Default value: `'SERVER'`

##### <a name="database"></a>`database`

Data type: `String[1,128]`

The name of the database the role exists on when specifying `type => 'DATABASE'`. Defaults to 'master'

Default value: `'master'`

##### <a name="permissions"></a>`permissions`

Data type: `Optional[Hash]`

A hash of permissions that should be managed for the role.
Valid keys are 'GRANT', 'GRANT_WITH_OPTION', 'DENY' or 'REVOKE'.
Valid values must be an array of Strings i.e. {'GRANT' => ['CONNECT', 'CREATE ANY DATABASE'] }

Default value: `{ }`

##### <a name="members"></a>`members`

Data type: `Array[String]`

An array of users/logins that should be a member of the role

Default value: `[]`

##### <a name="members_purge"></a>`members_purge`

Data type: `Boolean`

Whether we should purge any members not listed in the members parameter. Default: false

Default value: ``false``

### <a name="sqlserverrolepermissions"></a>`sqlserver::role::permissions`

Requirement/Dependencies:
Requires defined type {sqlserver::config} in order to execute against the SQL Server instance

#### Parameters

The following parameters are available in the `sqlserver::role::permissions` defined type:

* [`role`](#role)
* [`permissions`](#permissions)
* [`state`](#state)
* [`with_grant_option`](#with_grant_option)
* [`type`](#type)
* [`database`](#database)
* [`instance`](#instance)

##### <a name="role"></a>`role`

Data type: `String[1,128]`

The name of the role for which the permissions will be manage.

##### <a name="permissions"></a>`permissions`

Data type: `Array[String[4,128]]`

An array of permissions you want manged for the given role

##### <a name="state"></a>`state`

Data type: `Pattern[/(?i)^(GRANT|REVOKE|DENY)$/]`

The state you would like the permission in.
Accepts 'GRANT', 'DENY', 'REVOKE'.
Please note that REVOKE equates to absent and will default to database and system level permissions.

Default value: `'GRANT'`

##### <a name="with_grant_option"></a>`with_grant_option`

Data type: `Optional[Boolean]`

Whether to give the role the option to grant this permission to other principal objects, accepts true or false, defaults to false

Default value: ``false``

##### <a name="type"></a>`type`

Data type: `Enum['SERVER','DATABASE']`

Whether the Role is `SERVER` or `DATABASE`

Default value: `'SERVER'`

##### <a name="database"></a>`database`

Data type: `String[1,128]`

The name of the database the role exists on when specifying `type => 'DATABASE'`. Defaults to 'master'

Default value: `'master'`

##### <a name="instance"></a>`instance`

Data type: `String[1,16]`

The name of the instance where the role and database exists. Defaults to 'MSSQLSERVER'

Default value: `'MSSQLSERVER'`

### <a name="sqlserversp_configure"></a>`sqlserver::sp_configure`

Required Dependencies:
Requires defined type {sqlserver::config} in order to execute against the SQL Server instance

 @see http://msdn.microsoft.com/en-us/library/ms176069.aspx Reconfigure Explanation
 @see http://msdn.microsoft.com/en-us/library/ms189631.aspx Server Configuration Options

#### Parameters

The following parameters are available in the `sqlserver::sp_configure` defined type:

* [`config_name`](#config_name)
* [`value`](#value)
* [`instance`](#instance)
* [`reconfigure`](#reconfigure)
* [`with_override`](#with_override)
* [`restart`](#restart)

##### <a name="config_name"></a>`config_name`

Data type: `Pattern['^\w+']`

The config name found within sys.configurations that you would like to update

Default value: `$title`

##### <a name="value"></a>`value`

Data type: `Integer`

The value you would like to change to for the given `config_name`, must be an integer value

##### <a name="instance"></a>`instance`

Data type: `String[1,16]`

The name of the instance you would like to manage against

Default value: `'MSSQLSERVER'`

##### <a name="reconfigure"></a>`reconfigure`

Data type: `Boolean`

If you would like to run RECONFIGURE against the server after updating the value, defaults to true

Default value: ``true``

##### <a name="with_override"></a>`with_override`

Data type: `Boolean`

This pertains tot he `reconfigure` in which you would want to override or force the reconfigure, defaults to false

Default value: ``false``

##### <a name="restart"></a>`restart`

Data type: `Boolean`

Will ensure service resource and notify if changes occur for a restart

Default value: ``false``

### <a name="sqlserveruser"></a>`sqlserver::user`

Requirement/Dependencies:
Requires defined type {sqlserver::config} in order to execute against the SQL Server instance

#### Examples

##### 

```puppet
sqlserver::user{'myUser':
  database  => 'loggingDatabase',
  login     => 'myUser',
}
```

#### Parameters

The following parameters are available in the `sqlserver::user` defined type:

* [`user`](#user)
* [`database`](#database)
* [`ensure`](#ensure)
* [`default_schema`](#default_schema)
* [`instance`](#instance)
* [`login`](#login)
* [`password`](#password)
* [`permissions`](#permissions)

##### <a name="user"></a>`user`

Data type: `String[1]`

The username you want to manage, defaults to the title

Default value: `$title`

##### <a name="database"></a>`database`

Data type: `String[1,128]`

The database you want the user to be created as

##### <a name="ensure"></a>`ensure`

Data type: `Enum['present', 'absent']`

Ensure present or absent

Default value: `'present'`

##### <a name="default_schema"></a>`default_schema`

Data type: `Optional[String]`

SQL schema you would like to default to, typically 'dbo'

Default value: ``undef``

##### <a name="instance"></a>`instance`

Data type: `String[1,16]`

The named instance you want to manage against

Default value: `'MSSQLSERVER'`

##### <a name="login"></a>`login`

Data type: `Optional[String[1]]`

The login to associate the user with, by default SQL Server will assume user and login match if left empty

Default value: ``undef``

##### <a name="password"></a>`password`

Data type: `Optional[String[1,128]]`

The password for the user, can only be used when the database is a contained database.

Default value: ``undef``

##### <a name="permissions"></a>`permissions`

Data type: `Optional[Hash]`

A hash of permissions that should be managed for the user.
Valid keys are 'GRANT', 'GRANT_WITH_OPTION', 'DENY' or 'REVOKE'.
Valid values must be an array of Strings i.e. {'GRANT' => ['SELECT', 'INSERT'] }

Default value: `{ }`

### <a name="sqlserveruserpermissions"></a>`sqlserver::user::permissions`

Requirement/Dependencies:
Requires defined type {sqlserver::config} in order to execute against the SQL Server instance

#### Parameters

The following parameters are available in the `sqlserver::user::permissions` defined type:

* [`user`](#user)
* [`database`](#database)
* [`permissions`](#permissions)
* [`state`](#state)
* [`with_grant_option`](#with_grant_option)
* [`instance`](#instance)

##### <a name="user"></a>`user`

Data type: `String[1,128]`

The username for which the permission will be manage.

##### <a name="database"></a>`database`

Data type: `String[1,128]`

The databaser you would like the permission managed on.

Default value: `'master'`

##### <a name="permissions"></a>`permissions`

Data type: `Array[String[4,128]]`

An array of permissions you would like managed. i.e. ['SELECT', 'INSERT', 'UPDATE', 'DELETE']

##### <a name="state"></a>`state`

Data type: `Pattern[/(?i)^(GRANT|REVOKE|DENY)$/]`

The state you would like the permission in.
Accepts 'GRANT', 'DENY', 'REVOKE'.
Please note that REVOKE equates to absent and will default to database and system level permissions.

Default value: `'GRANT'`

##### <a name="with_grant_option"></a>`with_grant_option`

Data type: `Optional[Boolean]`

Whether to give the user the option to grant this permission to other users, accepts true or false, defaults to false

Default value: ``false``

##### <a name="instance"></a>`instance`

Data type: `String[1,16]`

The name of the instance where the user and database exists. Defaults to 'MSSQLSERVER'

Default value: `'MSSQLSERVER'`

## Resource types

### <a name="sqlserver_features"></a>`sqlserver_features`

Due to our prefetch and unaware of what name the user will provide we munge the value to meet our expecations.

#### Properties

The following properties are available in the `sqlserver_features` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

##### `features`

Valid values: `Tools`, `BC`, `Conn`, `SSMS`, `ADV_SSMS`, `SDK`, `IS`, `MDS`, `BOL`, `DREPLAY_CTLR`, `DREPLAY_CLT`, `DQC`

Specifies features to install, uninstall, or upgrade. The list of top-level features include
BC, Conn, SSMS, ADV_SSMS, SDK, IS and MDS.

#### Parameters

The following parameters are available in the `sqlserver_features` type.

* [`install_switches`](#install_switches)
* [`is_svc_account`](#is_svc_account)
* [`is_svc_password`](#is_svc_password)
* [`name`](#name)
* [`pid`](#pid)
* [`provider`](#provider)
* [`source`](#source)
* [`windows_feature_source`](#windows_feature_source)

##### <a name="install_switches"></a>`install_switches`

A hash of switches you want to pass to the installer

##### <a name="is_svc_account"></a>`is_svc_account`

Either domain user name or system account. Defaults to "NT AUTHORITY\NETWORK SERVICE"

##### <a name="is_svc_password"></a>`is_svc_password`

Password for domain user.

##### <a name="name"></a>`name`

namevar

Due to our prefetch and unaware of what name the user will provide we munge the value to meet our expecations.

##### <a name="pid"></a>`pid`

Specify the SQL Server product key to configure which edition you would like to use.

##### <a name="provider"></a>`provider`

The specific backend to use for this `sqlserver_features` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

##### <a name="source"></a>`source`

Location of the source files.

##### <a name="windows_feature_source"></a>`windows_feature_source`

Specify the location of the Windows Feature source files.  This may be needed to install the .Net Framework.
See https://support.microsoft.com/en-us/kb/2734782 for more information.

### <a name="sqlserver_instance"></a>`sqlserver_instance`

Namevar

#### Properties

The following properties are available in the `sqlserver_instance` type.

##### `ensure`

Valid values: `present`, `absent`

The basic property that the resource should be in.

Default value: `present`

##### `features`

Valid values: `SQL`, `SQLEngine`, `Replication`, `FullText`, `DQ`, `AS`, `RS`, `POLYBASE`, `ADVANCEDANALYTICS`

Specifies features to install, uninstall, or upgrade. The list of top-level features include
SQLEngine, Replication, FullText, DQ AS, and RS.

#### Parameters

The following parameters are available in the `sqlserver_instance` type.

* [`agt_svc_account`](#agt_svc_account)
* [`agt_svc_password`](#agt_svc_password)
* [`as_svc_account`](#as_svc_account)
* [`as_svc_password`](#as_svc_password)
* [`as_sysadmin_accounts`](#as_sysadmin_accounts)
* [`install_switches`](#install_switches)
* [`name`](#name)
* [`pid`](#pid)
* [`polybase_svc_account`](#polybase_svc_account)
* [`polybase_svc_password`](#polybase_svc_password)
* [`provider`](#provider)
* [`rs_svc_account`](#rs_svc_account)
* [`rs_svc_password`](#rs_svc_password)
* [`sa_pwd`](#sa_pwd)
* [`security_mode`](#security_mode)
* [`source`](#source)
* [`sql_svc_account`](#sql_svc_account)
* [`sql_svc_password`](#sql_svc_password)
* [`sql_sysadmin_accounts`](#sql_sysadmin_accounts)
* [`windows_feature_source`](#windows_feature_source)

##### <a name="agt_svc_account"></a>`agt_svc_account`

Either domain user name or system account

##### <a name="agt_svc_password"></a>`agt_svc_password`

Password for domain user name. Not required for system account

##### <a name="as_svc_account"></a>`as_svc_account`

The account used by the Analysis Services service.

##### <a name="as_svc_password"></a>`as_svc_password`

The password for the Analysis Services service account.

##### <a name="as_sysadmin_accounts"></a>`as_sysadmin_accounts`

Specifies the list of administrator accounts to provision.

##### <a name="install_switches"></a>`install_switches`

A hash of switches you want to pass to the installer

##### <a name="name"></a>`name`

namevar

Namevar

##### <a name="pid"></a>`pid`

Specify the SQL Server product key to configure which edition you would like to use.

##### <a name="polybase_svc_account"></a>`polybase_svc_account`

The account used by the Polybase Engine service. Only applicable for SQL Server 2016+.

##### <a name="polybase_svc_password"></a>`polybase_svc_password`

The password for the Polybase Engine service account. Only applicable for SQL Server 2016+.

##### <a name="provider"></a>`provider`

The specific backend to use for this `sqlserver_instance` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

##### <a name="rs_svc_account"></a>`rs_svc_account`

Specify the service account of the report server. This value is required.
If you omit this value, Setup will use the default built-in account for
the current operating system (either NetworkService or LocalSystem).
If you specify a domain user account, the domain must be under 254 characters
and the user name must be under 20 characters. The account name cannot contain the
following characters: " / \ [ ] : ; | = , + * ? < >

##### <a name="rs_svc_password"></a>`rs_svc_password`

Specify a strong password for the account. A strong password is at least 8 characters and
includes a combination of upper and lower case alphanumeric characters and at least one symbol
character. Avoid spelling an actual word or name that might be listed in a dictionary.

##### <a name="sa_pwd"></a>`sa_pwd`

Required when :security_mode => "SQL"

##### <a name="security_mode"></a>`security_mode`

Valid values: `SQL`

Specifies the security mode for SQL Server.
If this parameter is not supplied, then Windows-only authentication mode is supported.
Supported value: SQL

##### <a name="source"></a>`source`

Location of source files.

##### <a name="sql_svc_account"></a>`sql_svc_account`

Account for SQL Server service: Domain\User or system account.

##### <a name="sql_svc_password"></a>`sql_svc_password`

A SQL Server service password is required only for a domain account.

##### <a name="sql_sysadmin_accounts"></a>`sql_sysadmin_accounts`

Windows account(s) to provision as SQL Server system administrators.

##### <a name="windows_feature_source"></a>`windows_feature_source`

Specify the location of the Windows Feature source files.  This may be needed to install the .Net Framework.
See https://support.microsoft.com/en-us/kb/2734782 for more information.

### <a name="sqlserver_tsql"></a>`sqlserver_tsql`

Namevar

#### Properties

The following properties are available in the `sqlserver_tsql` type.

##### `onlyif`

SQL Query to run and only run if exits with non-zero

##### `returns`

Returns the result of the executed command

Default value: `0`

#### Parameters

The following parameters are available in the `sqlserver_tsql` type.

* [`command`](#command)
* [`database`](#database)
* [`instance`](#instance)
* [`name`](#name)
* [`provider`](#provider)

##### <a name="command"></a>`command`

command to run against an instance with the authenticated credentials used in sqlserver::config

##### <a name="database"></a>`database`

initial database to connect to during query execution

Default value: `master`

##### <a name="instance"></a>`instance`

requires the usage of sqlserver::config with the user and password

##### <a name="name"></a>`name`

namevar

Namevar

##### <a name="provider"></a>`provider`

The specific backend to use for this `sqlserver_tsql` resource. You will seldom need to specify this --- Puppet will
usually discover the appropriate provider for your platform.

## Functions

### <a name="sqlserver_is_domain_or_local_user"></a>`sqlserver_is_domain_or_local_user`

Type: Ruby 3.x API

The sqlserver_is_domain_or_local_user function.

#### `sqlserver_is_domain_or_local_user()`

The sqlserver_is_domain_or_local_user function.

Returns: `Boolean` Returns true is the username is for local/domain.

### <a name="sqlserver_upcase"></a>`sqlserver_upcase`

Type: Ruby 3.x API

The sqlserver_upcase function.

#### `sqlserver_upcase()`

The sqlserver_upcase function.

Returns: `Any` Upcase values

### <a name="sqlserver_validate_hash_uniq_values"></a>`sqlserver_validate_hash_uniq_values`

Type: Ruby 3.x API

The sqlserver_validate_hash_uniq_values function.

#### `sqlserver_validate_hash_uniq_values()`

The sqlserver_validate_hash_uniq_values function.

Returns: `String` Returns the arguments or a message with the duplicate values.

### <a name="sqlserver_validate_instance_name"></a>`sqlserver_validate_instance_name`

Type: Ruby 3.x API

The sqlserver_validate_instance_name function.

#### `sqlserver_validate_instance_name()`

The sqlserver_validate_instance_name function.

Returns: `Any` Error if not a valid instance name.

### <a name="sqlserver_validate_range"></a>`sqlserver_validate_range`

Type: Ruby 3.x API

The sqlserver_validate_range function.

#### `sqlserver_validate_range()`

The sqlserver_validate_range function.

Returns: `Any` Error if value is not between range

### <a name="sqlserver_validate_size"></a>`sqlserver_validate_size`

Type: Ruby 3.x API

The sqlserver_validate_size function.

#### `sqlserver_validate_size()`

The sqlserver_validate_size function.

Returns: `Any` Error if not a valid size value

## Tasks

### <a name="get_sql_logins"></a>`get_sql_logins`

Retrieve information about the logins configured for a SQL Server instance.

**Supports noop?** false

#### Parameters

##### `instance_name`

Data type: `Optional[Variant[Array[String], String]]`

The name of the SQL Instance running on the machine to connect to. Leave blank for the default instance of MSSQLSERVER

##### `login_name`

Data type: `Optional[Variant[Array[String], String]]`

The name of a particular login to search for. You can use partial names and any pattern that will work with the PowerShell '-match' operator.

##### `exact_match`

Data type: `Optional[Boolean]`

If set to true it will force names passed to the LoginName parameter to be an exact match to a SQL Login to pass the filter.

##### `detailed`

Data type: `Optional[Boolean]`

Return more detailed information from the server instead of the default summary information

### <a name="get_sqlagent_jobs"></a>`get_sqlagent_jobs`

Return information about SQL Agent jobs and job steps.

**Supports noop?** false

#### Parameters

##### `instance_name`

Data type: `Optional[Variant[Array[String],String]]`

The instance to get job information from

##### `job_name`

Data type: `Optional[Variant[Array[String],String]]`

The name or the pattern to match of the job to search for.

##### `exact_match`

Data type: `Optional[Boolean]`

Use only exact name matches for the job_name parameter instead of the default fuzzy matching.

### <a name="set_sql_logins"></a>`set_sql_logins`

Set IsDisabled, and Password properties of a SQL Login

**Supports noop?** true

#### Parameters

##### `instance_name`

Data type: `Optional[Variant[Array[String],String]]`

Instance that has the login to be modified. Leave blank for default instance.

##### `login_name`

Data type: `Variant[Array[String],String]`

Name of the login to modify. Matches are exact only by default.

##### `fuzzy_match`

Data type: `Optional[Boolean]`

Allow -match operator matches on $login_name so that inputs like 'sql' will match any login with 'sql' anywhere in the name.

##### `enabled`

Data type: `Optional[Boolean]`

Enable or disable an account. Set this to false to disable the account.

##### `password`

Data type: `Optional[String]`

The password to set for an account.

### <a name="start_sql_agent_job"></a>`start_sql_agent_job`

Start SQL Agent jobs on a server. You can start at a specified job step number (zero based indexes), and you can either wait on the job to complete, or return immediately.

**Supports noop?** false

#### Parameters

##### `instance_name`

Data type: `Optional[Variant[Array[String],String]]`

The instance to start a job on.

##### `job_name`

Data type: `Variant[Array[String],String]`

The name of the job to start.

##### `fuzzy_match`

Data type: `Optional[Boolean]`

Turn the job_name parameter into a pattern to match using the PowerShell -match operator.

##### `step`

Data type: `Optional[Integer]`

The zero based index number of the jop step to start from. Defaults to zero.

##### `wait`

Data type: `Optional[Boolean]`

Wait for all jobs started to complete before returning data. Defaults to false such that the task will return immediately indicating only that the job was started.

